#!/usr/bin/env python
"""Test the tei parser"""
import re
from pathlib import Path

import pytest

import colrev.env.environment_manager
import colrev.env.tei_parser
import colrev.exceptions as colrev_exceptions
from colrev.constants import ENTRYTYPES
from colrev.constants import Fields
from colrev.constants import RecordState

# pylint: disable=line-too-long
# pylint: disable=too-many-lines


@pytest.fixture(scope="module")
def script_loc(request) -> Path:  # type: ignore
    """Return the directory of the currently running test script"""

    return Path(request.fspath).parent


@pytest.fixture(scope="module")
def tei_doc(script_loc) -> colrev.env.tei_parser.TEIParser:  # type: ignore
    """Return the tei_doc"""

    tei_file = script_loc.parent.joinpath("data/WagnerLukyanenkoParEtAl2022.tei.xml")

    environment_manager = colrev.env.environment_manager.EnvironmentManager()

    tei_doc = colrev.env.tei_parser.TEIParser(
        environment_manager=environment_manager, tei_path=tei_file
    )
    return tei_doc


def test_tei_creation(script_loc, base_repo_review_manager) -> None:  # type: ignore
    """Test the tei"""
    if base_repo_review_manager.in_ci_environment():
        return
    tei_file = script_loc.parent.joinpath("data/WagnerLukyanenkoParEtAl2022.tei.xml")
    pdf_path = script_loc.parent.joinpath("data/WagnerLukyanenkoParEtAl2022.pdf")

    tei_file.unlink(missing_ok=True)

    environment_manager = colrev.env.environment_manager.EnvironmentManager()

    colrev.env.tei_parser.TEIParser(
        environment_manager=environment_manager, pdf_path=pdf_path, tei_path=tei_file
    )

    with open(tei_file) as file:
        tei_content = file.read()

    tei_content = re.sub(
        r'(ident="GROBID" when=")[^"]+(">)', r"\g<1>NA\g<2>", tei_content
    )

    with open(tei_file, "w") as file:
        file.write(tei_content)


def test_tei_version(tei_doc) -> None:  # type: ignore
    """Test the tei version"""
    assert "0.8.1" == tei_doc.get_grobid_version()


def test_tei_get_metadata(tei_doc) -> None:  # type: ignore
    """Test the tei version"""
    assert (
        "Artificial intelligence (AI) is beginning to transform traditional research practices in many areas. In this context, literature reviews stand out because they operate on large and rapidly growing volumes of documents, that is, partially structured (meta)data, and pervade almost every type of paper published in information systems research or related social science disciplines. To familiarize researchers with some of the recent trends in this area, we outline how AI can expedite individual steps of the literature review process. Considering that the use of AI in this context is in an early stage of development, we propose a comprehensive research agenda for AI-based literature reviews (AILRs) in our field. With this agenda, we would like to encourage design science research and a broader constructive discourse on shaping the future of AILRs in research."
        == tei_doc.get_abstract()
    )

    # Note : Journal extraction not (yet) supported well
    # Did not find a journal paper where the journal was extracted correctly
    assert {
        Fields.ENTRYTYPE: ENTRYTYPES.MISC,
        Fields.AUTHOR: "Wagner, Gerit and Lukyanenko, Roman and Par, Guy and Paré, Guy",
        Fields.DOI: "10.1177/02683962211048201",
        Fields.TITLE: "Debates and Perspectives Paper",
    } == tei_doc.get_metadata()

    assert [
        "Artificial intelligence",
        "machine learning",
        "natural language processing",
        "research data management",
        "data infrastructure",
        "automation",
        "literature review",
    ] == tei_doc.get_paper_keywords()

    assert [
        {"surname": "Wagner", "forename": "Gerit"},
        {"surname": "Lukyanenko", "forename": "Roman"},
        {
            "surname": "Par",
            "forename": "Guy",
            "emai": "guy.pare@hec.ca",
            "ORCID": "0000-0001-7425-1994",
        },
        {"surname": "Paré", "forename": "Guy"},
    ] == tei_doc.get_author_details()


def test_tei_reference_extraction(tei_doc) -> None:  # type: ignore
    """Test the tei extraction of references"""

    assert [
        {
            Fields.ID: "b0",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b0",
            Fields.AUTHOR: "Adams, C. E and Polzmacher, S. and Wolff, A.",
            Fields.TITLE: "Systematic reviews: work that needs to be done and not to be done",
            Fields.YEAR: "2013",
            Fields.JOURNAL: "Journal of Evidence-Based Medicine",
            Fields.VOLUME: "6",
            Fields.NUMBER: "4",
            Fields.PAGES: "232--235",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b1",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b1",
            Fields.AUTHOR: "Alter, S.",
            Fields.TITLE: "Architecture of Sysperanto: a model-based ontology of the is field",
            Fields.YEAR: "2005",
            Fields.JOURNAL: "Communications of the Association for Information Systems",
            Fields.VOLUME: "15",
            Fields.NUMBER: "1",
            Fields.PAGES: "1--40",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b2",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b2",
            Fields.AUTHOR: "Alvesson, M. and Sandberg, J.",
            Fields.TITLE: "Generating research questions through problematization",
            Fields.YEAR: "2011",
            Fields.JOURNAL: "Academy of Management Review",
            Fields.VOLUME: "36",
            Fields.NUMBER: "2",
            Fields.PAGES: "247--271",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b3",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b3",
            Fields.AUTHOR: "Al-Zubidy, A. and Carver, J. C and Hale, D. P",
            Fields.TITLE: "Vision for SLR tooling infrastructure: prioritizing value-added requirements",
            Fields.YEAR: "2017",
            Fields.JOURNAL: "Information and Software Technology",
            Fields.VOLUME: "91",
            Fields.PAGES: "72--81",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b4",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b4",
            Fields.AUTHOR: "Anderson, M. R and Antenucci, D. and Bittorf, V.",
            Fields.TITLE: "Brainwash: a data system for feature engineering",
            Fields.BOOKTITLE: "Proceedings of the biennial conference on innovative data systems research",
            Fields.YEAR: "2013",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b5",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b5",
            Fields.AUTHOR: "Antons, D. and Breidbach, C. F",
            Fields.TITLE: "Big data, big insights? Advancing service innovation and design with machine learning",
            Fields.YEAR: "2017",
            Fields.JOURNAL: "Journal of Service Research",
            Fields.VOLUME: "21",
            Fields.NUMBER: "1",
            Fields.PAGES: "17--39",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b6",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b6",
            Fields.AUTHOR: "Antons, D. and Breidbach, C. F and Joshi, A. M",
            Fields.TITLE: "Computational literature reviews: method, algorithms, and roadmap",
            Fields.YEAR: "2021",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b7",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b7",
            Fields.AUTHOR: "Avison, D. and Malaurent, J.",
            Fields.TITLE: "Is theory king?: questioning the theory fetish in Information Systems",
            Fields.YEAR: "2014",
            Fields.JOURNAL: "Journal of Information Technology",
            Fields.VOLUME: "29",
            Fields.NUMBER: "4",
            Fields.PAGES: "327--336",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b8",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b8",
            Fields.AUTHOR: "Balducci, B. and Marinova, D.",
            Fields.TITLE: "Unstructured data in marketing",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Journal of the Academy of Marketing Science",
            Fields.VOLUME: "46",
            Fields.NUMBER: "4",
            Fields.PAGES: "557--590",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b9",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b9",
            Fields.AUTHOR: "Bandara, W. and Furtmueller, E. and Gorbacheva, E.",
            Fields.TITLE: "Achieving rigour in literature reviews: insights from qualitative data analysis and tool-support",
            Fields.YEAR: "2015",
            Fields.JOURNAL: "Communications of the Association for Information Systems",
            Fields.VOLUME: "34",
            Fields.NUMBER: "8",
            Fields.PAGES: "154--204",
            Fields.NR_INTEXT_CITATIONS: 0,
        },
        {
            Fields.ID: "b10",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b10",
            Fields.AUTHOR: "Batini, C. and Rula, A. and Scannapieco, M.",
            Fields.TITLE: "From data quality to big data quality",
            Fields.YEAR: "2015",
            Fields.JOURNAL: "Journal of Database Management",
            Fields.VOLUME: "26",
            Fields.NUMBER: "1",
            Fields.PAGES: "60--82",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b11",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b11",
            Fields.AUTHOR: "Bax, L. and Yu, L-M and Ikeda, N.",
            Fields.TITLE: "A systematic comparison of software dedicated to meta-analysis of causal studies",
            Fields.YEAR: "2007",
            Fields.JOURNAL: "BMC Medical Research Methodology",
            Fields.VOLUME: "7",
            Fields.NUMBER: "1",
            Fields.PAGES: "1--9",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b12",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b12",
            Fields.AUTHOR: "Beller, E. and Clark, J. and Tsafnat, G.",
            Fields.TITLE: "Making progress with the automation of systematic reviews: principles of the International Collaboration for the Automation of Systematic Reviews (ICASR)",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Systematic Reviews",
            Fields.VOLUME: "7",
            Fields.NUMBER: "1",
            Fields.PAGES: "1--7",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b13",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b13",
            Fields.AUTHOR: "Berente, N. and Seidel, S. and Safadi, H.",
            Fields.TITLE: "Research commentarydata-driven computationally intensive theory development",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "Information Systems Research",
            Fields.VOLUME: "30",
            Fields.NUMBER: "1",
            Fields.PAGES: "50--64",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b14",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b14",
            Fields.AUTHOR: "Boell, S. K and Wang, B ; Kk and Wei, W. W and Huang,  and Lee, ",
            Fields.TITLE: "wwwlitbaskets.io, an IT artifact supporting exploratory literature searches for Information Systems research",
            Fields.BOOKTITLE: "Proceedings of the Pacific Asia conference on information systems",
            Fields.YEAR: "2019",
            Fields.PAGES: "8--12",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b15",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b15",
            Fields.AUTHOR: "Boh, Burton-Jones A and Oborn, W. F and , E.",
            Fields.TITLE: "Editor's Comments: advancing research transparency at MIS quarterly: a pluralistic approach",
            Fields.YEAR: "2021",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "45",
            Fields.NUMBER: "2",
            Fields.PAGES: "iii--xviii",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b16",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b16",
            Fields.AUTHOR: "Carver, J. C and Hassler, E. and Hernandes, E.",
            Fields.TITLE: "Identifying barriers to the systematic literature review process",
            Fields.BOOKTITLE: "ternational symposium on empirical software engineering and measurement",
            Fields.YEAR: "2013",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b17",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b17",
            Fields.AUTHOR: "Castellanos, A. and Castillo, A. and Tremblay, M. C",
            Fields.TITLE: "Improving machine learning performance using conceptual modeling",
            Fields.BOOKTITLE: "AAAI spring symposium: Combining machine learning with knowledge engineering",
            Fields.YEAR: "2021",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b18",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b18",
            Fields.AUTHOR: "Castelvecchi, D.",
            Fields.TITLE: "Can we open the black box of AI?",
            Fields.YEAR: "2016",
            Fields.JOURNAL: "Nature",
            Fields.VOLUME: "538",
            Fields.NUMBER: "7623",
            Fields.PAGES: "20--23",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b19",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b19",
            Fields.AUTHOR: "Choudhury, P. and Endres, Allen R and , M.",
            Fields.TITLE: "Developing theory using machine learning methods",
            Fields.YEAR: "2018",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b20",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b20",
            Fields.AUTHOR: "Cooper, H. M",
            Fields.TITLE: "Organizing knowledge syntheses: a taxonomy of literature reviews",
            Fields.YEAR: "1988",
            Fields.JOURNAL: "Knowledge in Society",
            Fields.VOLUME: "1",
            Fields.NUMBER: "1",
            Fields.PAGES: "104--126",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b21",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b21",
            Fields.AUTHOR: "Cram, W. A and Templier, M. and Templier, M.",
            Fields.TITLE: "Re)considering the concept of literature review reproducibility",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "Journal of the Association for Information Systems",
            Fields.VOLUME: "21",
            Fields.NUMBER: "5",
            Fields.PAGES: "1103--1114",
            Fields.NR_INTEXT_CITATIONS: 4,
        },
        {
            Fields.ID: "b22",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b22",
            Fields.AUTHOR: "Dalgali, A. and Crowston, K.",
            Fields.TITLE: "Sharing open deep learning models",
            Fields.BOOKTITLE: "Proceedings of the hawaii international conference on system sciences, Grand",
            Fields.YEAR: "2019",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b23",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b23",
            Fields.AUTHOR: "Dang, Y. and Zhang, Y. and Chen, H.",
            Fields.TITLE: "Arizona literature mapper: an integrated approach to monitor and analyze global bioterrorism research literature",
            Fields.YEAR: "2009",
            Fields.JOURNAL: "Journal of the American Society for Information Science and Technology",
            Fields.VOLUME: "60",
            Fields.NUMBER: "7",
            Fields.PAGES: "1466--1485",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b24",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b24",
            Fields.AUTHOR: "Dörpinghaus, J. and , Stefan A",
            Fields.TITLE: "Knowledge extraction and applications utilizing context data in knowledge graphs",
            Fields.BOOKTITLE: "Proceedings of the federated conference on computer science and information systems",
            Fields.YEAR: "2019",
            Fields.PAGES: "265--272",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b25",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b25",
            Fields.AUTHOR: "Dubé, L. and Paré, G.",
            Fields.TITLE: "Rigor in information systems positivist case research: current practices, trends, and recommendations",
            Fields.YEAR: "2003",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "27",
            Fields.NUMBER: "4",
            Fields.PAGES: "597--636",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b26",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b26",
            Fields.AUTHOR: "Duboue, P.",
            Fields.TITLE: "The Art of Feature Engineering: Essentials for Machine Learning",
            Fields.BOOKTITLE: "The Art of Feature Engineering: Essentials for Machine Learning",
            Fields.YEAR: "2020",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b27",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b27",
            Fields.AUTHOR: "Eisenstein, J.",
            Fields.TITLE: "Introduction to Natural Language Processing",
            Fields.YEAR: "2019",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b28",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b28",
            Fields.AUTHOR: "Endicott, J. and Larsen, K. and Lukyanenko, R.",
            Fields.TITLE: "Integrating scientific research: Theory and design of discovering similar constructs",
            Fields.YEAR: "2017",
            Fields.BOOKTITLE: "AIS SIGSAND Symposium",
            Fields.PAGES: "1--7",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b29",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b29",
            Fields.AUTHOR: "Evans, J. A and Foster, J. G",
            Fields.YEAR: "2011",
            Fields.JOURNAL: "Metaknowledge. Science",
            Fields.VOLUME: "331",
            Fields.NUMBER: "11",
            Fields.PAGES: "721--725",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b30",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b30",
            Fields.AUTHOR: "Germonprez, M. and Hovorka, D. and Hovorka, D.",
            Fields.TITLE: "A theory of tailorable technology design",
            Fields.YEAR: "2007",
            Fields.JOURNAL: "Journal of the Association for Information Systems",
            Fields.VOLUME: "8",
            Fields.NUMBER: "6",
            Fields.PAGES: "351--367",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b31",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b31",
            Fields.AUTHOR: "Gleasure, R.",
            Fields.TITLE: "Conceptual design science research? How and why untested meta-artifacts have a place in IS",
            Fields.BOOKTITLE: "Proceedings of the international conference on design science research in information systems and technology",
            Fields.YEAR: "2014",
            Fields.PAGES: "99--114",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b32",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b32",
            Fields.AUTHOR: "Goodfellow, I. and Bengio, Y. and Courville, A.",
            Fields.TITLE: "Deep Learning, Adaptive Computation and Machine Learning Series",
            Fields.YEAR: "2016",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b33",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b33",
            Fields.AUTHOR: "Gunning, D. and Aha, D.",
            Fields.TITLE: "DARPA's Explainable Artificial Intelligence (XAI) Program",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "AI Magazine",
            Fields.VOLUME: "40",
            Fields.NUMBER: "2",
            Fields.PAGES: "44--58",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b34",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b34",
            Fields.AUTHOR: "Hake, P. and Fettke, P. and Neumann, G.",
            Fields.TITLE: "Extracting business objects and activities from labels of German process models",
            Fields.BOOKTITLE: "Proceedings of the international conference on design science research in information system and technology",
            Fields.YEAR: "2017",
            Fields.PAGES: "21--38",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b35",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b35",
            Fields.AUTHOR: "Harrison, H. and Griffin, S. J and Kuhn, I.",
            Fields.TITLE: "Software tools to support title and abstract screening for systematic reviews in healthcare: an evaluation",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "BMC Medical Research Methodology",
            Fields.VOLUME: "20",
            Fields.NUMBER: "7",
            Fields.PAGES: "1--12",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b36",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b36",
            Fields.AUTHOR: "Hartling, L. and Ospina, M. and Liang, Y.",
            Fields.TITLE: "Risk of bias versus quality assessment of randomised controlled trials: cross sectional study",
            Fields.YEAR: "2009",
            Fields.JOURNAL: "British Medical Journal",
            Fields.VOLUME: "339",
            Fields.NUMBER: "1",
            Fields.PAGES: "1--6",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b37",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b37",
            Fields.AUTHOR: "Hassan, N. R and Prester, J. and Wagner, G.",
            Fields.TITLE: "Seeking out clear and unique Information Systems Concepts: a natural language processing approach",
            Fields.BOOKTITLE: "Proceedings of the European conference on information systems",
            Fields.YEAR: "2020",
            Fields.PAGES: "15--17",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b38",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b38",
            Fields.AUTHOR: "Higgins, Jpt and Green, S.",
            Fields.TITLE: "Cochrane Handbook for Systematic Reviews of Interventions",
            Fields.YEAR: "2008",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b39",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b39",
            Fields.AUTHOR: "Holzinger, A.",
            Fields.TITLE: "Interactive machine learning for health informatics: when do we need the human-in-the-loop?",
            Fields.YEAR: "2016",
            Fields.JOURNAL: "Brain Informatics",
            Fields.VOLUME: "3",
            Fields.NUMBER: "2",
            Fields.PAGES: "119--131",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b40",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b40",
            Fields.AUTHOR: "Hossain, M. S and Gresock, J. and Edmonds, Y.",
            Fields.TITLE: "Connecting the dots between PubMed abstracts",
            Fields.YEAR: "2012",
            Fields.JOURNAL: "PLoS One",
            Fields.VOLUME: "7",
            Fields.NUMBER: "1",
            Fields.PAGES: "1--23",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b41",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b41",
            Fields.AUTHOR: "Hunter, J. E and Schmidt, F. L",
            Fields.TITLE: "Methods of Meta-Analysis: Correcting Error and Bias in Research Findings. 2nd edition",
            Fields.BOOKTITLE: "Methods of Meta-Analysis: Correcting Error and Bias in Research Findings. 2nd edition",
            Fields.YEAR: "2014",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b42",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b42",
            Fields.AUTHOR: "Hutson, M.",
            Fields.TITLE: "Has artificial intelligence become alchemy?",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Science",
            Fields.VOLUME: "360",
            Fields.NUMBER: "6388",
            Fields.PAGES: "478--479",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b43",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b43",
            Fields.AUTHOR: "Hutson, M.",
            Fields.TITLE: "Artificial intelligence faces reproducibility crisis",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Science",
            Fields.VOLUME: "359",
            Fields.NUMBER: "6377",
            Fields.PAGES: "725--726",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b44",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b44",
            Fields.AUTHOR: "Jalali, S. and Wohlin, C.",
            Fields.TITLE: "Systematic literature studies: database searches vs. backward snowballing",
            Fields.BOOKTITLE: "Proceedings of the ACM-IEEE international symposium on empirical software engineering and measurement",
            Fields.YEAR: "2012",
            Fields.PAGES: "29--38",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b45",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b45",
            Fields.AUTHOR: "Johnson, C. D and Bauer, B. C and Niederman, F.",
            Fields.YEAR: "2019",
            Fields.TITLE: "The Automation of Management and Business Science. Academy of Management Perspectives",
            Fields.PAGES: "292--309",
            Fields.NR_INTEXT_CITATIONS: 4,
        },
        {
            Fields.ID: "b46",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b46",
            Fields.AUTHOR: "Jones, M.",
            Fields.TITLE: "How do we address the reproducibility crisis in artificial intelligence? Forbes",
            Fields.YEAR: "2018",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b47",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b47",
            Fields.AUTHOR: "Jonnalagadda, S. R and Goyal, P. and Huffman, M. D",
            Fields.TITLE: "Automating data extraction in systematic reviews: a systematic review",
            Fields.YEAR: "2015",
            Fields.JOURNAL: "Systematic Reviews",
            Fields.VOLUME: "4",
            Fields.NUMBER: "1",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b48",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b48",
            Fields.AUTHOR: "Kallinikos, J. and Tempini, N.",
            Fields.TITLE: "Patient data as medical facts: social media practices as a foundation for medical knowledge creation",
            Fields.YEAR: "2014",
            Fields.JOURNAL: "Information Systems Research",
            Fields.VOLUME: "25",
            Fields.NUMBER: "4",
            Fields.PAGES: "817--833",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b49",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b49",
            Fields.AUTHOR: "Kenett, R. S and Shmueli, G.",
            Fields.TITLE: "Information Quality: The Potential of Data and Analytics to Generate Knowledge",
            Fields.YEAR: "2016",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b50",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b50",
            Fields.AUTHOR: "King, R. D and Rowland, J. and Oliver, S. G",
            Fields.TITLE: "The automation of science",
            Fields.YEAR: "2009",
            Fields.JOURNAL: "Science",
            Fields.VOLUME: "324",
            Fields.NUMBER: "5923",
            Fields.PAGES: "85--89",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b51",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b51",
            Fields.AUTHOR: "Kitchenham, B. A and Charters, S.",
            Fields.TITLE: "Guidelines for performing systematic literature reviews in software engineering",
            Fields.YEAR: "2007",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b52",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b52",
            Fields.AUTHOR: "Klein, H. K and Myers, M. D",
            Fields.TITLE: "A set of principles for conducting and evaluating interpretive field studies in information systems",
            Fields.YEAR: "1999",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "23",
            Fields.NUMBER: "1",
            Fields.PAGES: "67--94",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b53",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b53",
            Fields.AUTHOR: "Knight, W.",
            Fields.TITLE: "Darpa Is Funding Projects that Will Try to Open up AI's Black Boxes",
            Fields.YEAR: "2017",
            Fields.JOURNAL: "MIT Technology Review",
            Fields.VOLUME: "25",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b54",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b54",
            Fields.AUTHOR: "Kobayashi, V. B and Mol, S. T and Berkers, H. A",
            Fields.TITLE: "Text mining in organizational research",
            Fields.YEAR: "2017",
            Fields.JOURNAL: "Organizational Research Methods",
            Fields.VOLUME: "21",
            Fields.NUMBER: "3",
            Fields.PAGES: "733--765",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b55",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b55",
            Fields.AUTHOR: "Kohl, C. and Mcintosh, E. J and Unger, S.",
            Fields.TITLE: "Online tools supporting the conduct and reporting of systematic reviews and systematic maps: a case study on cadima and review of existing tools",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Environmental Evidence",
            Fields.VOLUME: "7",
            Fields.NUMBER: "8",
            Fields.PAGES: "1--17",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b56",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b56",
            Fields.AUTHOR: "Kruse, Chandra and Seidel, L. and Vom Brocke, S. and , J.",
            Fields.TITLE: "Design archaeology: Generating design knowledge from real-world artifact design",
            Fields.BOOKTITLE: "Proceedings of the International Conference on Design Science Research in Information Systems and Technology",
            Fields.YEAR: "2019",
            Fields.PAGES: "32--45",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b57",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b57",
            Fields.AUTHOR: "Kunc, M. and Mortenson, M. J and Vidgen, R.",
            Fields.TITLE: "A computational literature review of the field of system dynamics from 1974 to 2017",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Journal of Simulation",
            Fields.VOLUME: "12",
            Fields.NUMBER: "2",
            Fields.PAGES: "115--127",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b58",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b58",
            Fields.AUTHOR: "Larsen, K. R and Bong, Bong Ch and , C. H",
            Fields.TITLE: "A tool for addressing construct identity in literature reviews and meta-analyses",
            Fields.YEAR: "2016",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "40",
            Fields.NUMBER: "3",
            Fields.PAGES: "529--551",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b59",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b59",
            Fields.AUTHOR: "Larsen, K. and Hovorka, D. and Dennis, A. R",
            Fields.TITLE: "Understanding the elephant: the discourse approach to boundary identification and corpus construction for theory review articles",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "Journal of the Association for Information Systems",
            Fields.VOLUME: "20",
            Fields.NUMBER: "7",
            Fields.PAGES: "887--928",
            Fields.NR_INTEXT_CITATIONS: 10,
        },
        {
            Fields.ID: "b60",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b60",
            Fields.AUTHOR: "Larsen, K. and Lukyanenko, R. and Muller, R.",
            Fields.TITLE: "Validity in design science research",
            Fields.BOOKTITLE: "Proceedings of the International Conference on Design Science Research in Information Systems and Technology",
            Fields.YEAR: "2020",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b61",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b61",
            Fields.AUTHOR: "Laurell, C. and Sandström, C. and Berthold, A.",
            Fields.TITLE: "Exploring barriers to adoption of virtual reality through social media analytics and machine learning-an assessment of technology, network, price and trialability",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "Journal of Business Research",
            Fields.VOLUME: "100",
            Fields.PAGES: "469--474",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b62",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b62",
            Fields.AUTHOR: "Leidner, D. E",
            Fields.TITLE: "Review and theory symbiosis: an introspective retrospective",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Journal of the Association for Information Systems",
            Fields.VOLUME: "19",
            Fields.NUMBER: "6",
            Fields.PAGES: "552--567",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b63",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b63",
            Fields.AUTHOR: "Li, J. and Larsen, K. and Abbasi, A.",
            Fields.TITLE: "TheoryOn: a design framework and system for unlocking behavioral knowledge through ontology learning",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "44",
            Fields.NUMBER: "4",
            Fields.PAGES: "1733--1772",
            Fields.NR_INTEXT_CITATIONS: 7,
        },
        {
            Fields.ID: "b64",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b64",
            Fields.AUTHOR: "Lindberg, A.",
            Fields.TITLE: "Developing theory through integrating human and machine pattern recognition",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "Journal of the Association for Information Systems",
            Fields.VOLUME: "21",
            Fields.NUMBER: "1",
            Fields.PAGES: "90--116",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b65",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b65",
            Fields.AUTHOR: "Lindzey, G. and Gilbert, D. and St, Fiske",
            Fields.YEAR: "1998",
            Fields.TITLE: "The Handbook of Social Psychology",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b66",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b66",
            Fields.AUTHOR: "Lukyanenko, R.",
            Fields.TITLE: "A journey to BSO: evaluating earlier and more recent ideas of Mario Bunge as a foundation for information and software development",
            Fields.BOOKTITLE: "Exploring Modeling Methods for Systems Analysis and Development",
            Fields.YEAR: "2020",
            Fields.PAGES: "1--15",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b67",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b67",
            Fields.AUTHOR: "Lukyanenko, R. and Parsons, J. and Parsons, J.",
            Fields.TITLE: "Research perspectives: design theory indeterminacy: what Is it, how can it be reduced, and why did the polar bear drown",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "Journal of the Association for Information Systems",
            Fields.VOLUME: "21",
            Fields.NUMBER: "5",
            Fields.PAGES: "1343--1369",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b68",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b68",
            Fields.AUTHOR: "Lukyanenko, R. and Parsons, J. and Wiersma, Y. F",
            Fields.TITLE: "Expecting the unexpected: effects of data collection design choices on the quality of crowdsourced user-generated content",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "43",
            Fields.NUMBER: "2",
            Fields.PAGES: "623--648",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b69",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b69",
            Fields.AUTHOR: "Maass, W. and Parsons, J. and Purao, S.",
            Fields.TITLE: "Data-driven meets theory-driven research in the era of big data: opportunities and challenges for Information Systems research",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Journal of the Association for Information Systems",
            Fields.VOLUME: "19",
            Fields.NUMBER: "12",
            Fields.PAGES: "1253--1273",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b70",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b70",
            Fields.AUTHOR: "Manning, C. D and Schütze, H.",
            Fields.TITLE: "Foundations of Statistical Natural Language Processing",
            Fields.YEAR: "1999",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b71",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b71",
            Fields.AUTHOR: "March, S. T and Gn, Allen",
            Fields.TITLE: "Toward a social ontology for conceptual modeling",
            Fields.YEAR: "2014",
            Fields.JOURNAL: "Communications of the Association for Information Systems",
            Fields.VOLUME: "34",
            Fields.NUMBER: "70",
            Fields.PAGES: "1347--1359",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b72",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b72",
            Fields.AUTHOR: "Marshall, I. J and Wallace, B. C",
            Fields.TITLE: "Toward systematic review automation: a practical guide to using machine learning tools in research synthesis",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "Systematic Reviews",
            Fields.VOLUME: "8",
            Fields.NUMBER: "1",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b73",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b73",
            Fields.AUTHOR: "Marshall, I. J and Kuiper, J. and Bc, Wallace",
            Fields.TITLE: "RobotReviewer: evaluation of a system for automatically assessing bias in clinical trials",
            Fields.YEAR: "2015",
            Fields.JOURNAL: "Journal of the American Medical Informatics Association",
            Fields.VOLUME: "23",
            Fields.NUMBER: "1",
            Fields.PAGES: "193--201",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b74",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b74",
            Fields.AUTHOR: "Mitchell, T. M",
            Fields.TITLE: "Machine learning",
            Fields.YEAR: "1997",
            Fields.PAGES: "870--877",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b75",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b75",
            Fields.AUTHOR: "Mortenson, M. J and Vidgen, R.",
            Fields.TITLE: "A computational literature review of the technology acceptance model",
            Fields.YEAR: "2016",
            Fields.JOURNAL: "International Journal of Information Management",
            Fields.VOLUME: "36",
            Fields.NUMBER: "6",
            Fields.PAGES: "1248--1259",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b76",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b76",
            Fields.AUTHOR: "Müller-Bloch, C. and Kranz, J.",
            Fields.TITLE: "A framework for rigorously identifying research gaps in qualitative literature reviews",
            Fields.BOOKTITLE: "Proceedings of the international conference on information systems",
            Fields.YEAR: "2015",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b77",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b77",
            Fields.AUTHOR: "Nakagawa, S. and Samarasinghe, G. and Haddaway, N. R",
            Fields.TITLE: "Research weaving: visualizing the future of research synthesis",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "Trends in Ecology & Evolution",
            Fields.VOLUME: "34",
            Fields.NUMBER: "3",
            Fields.PAGES: "224--238",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b78",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b78",
            Fields.AUTHOR: "Nelson, L. K",
            Fields.TITLE: "Computational grounded theory: a methodological framework",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "Sociological Methods & Research",
            Fields.VOLUME: "49",
            Fields.NUMBER: "1",
            Fields.PAGES: "3--42",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b79",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b79",
            Fields.AUTHOR: "O'connor, A. M and Tsafnat, G. and Gilbert, S. B",
            Fields.TITLE: "Moving toward the automation of the systematic review process: a summary of discussions at the second meeting of the International Collaboration for the Automation of Systematic Reviews (ICASR)",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Systematic Reviews",
            Fields.VOLUME: "7",
            Fields.NUMBER: "3",
            Fields.PAGES: "1--5",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b80",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b80",
            Fields.AUTHOR: "O'mara-Eves, A. and Thomas, J. and Mcnaught, J.",
            Fields.TITLE: "Using text mining for study identification in systematic reviews: a systematic review of current approaches",
            Fields.YEAR: "2015",
            Fields.JOURNAL: "Systematic Reviews",
            Fields.VOLUME: "4",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b81",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b81",
            Fields.AUTHOR: "Papaioannou, D. and Sutton, A. and Carroll, C.",
            Fields.TITLE: "Literature searching for social science systematic reviews: consideration of a range of search techniques",
            Fields.YEAR: "2009",
            Fields.JOURNAL: "Health Information & Libraries Journal",
            Fields.VOLUME: "27",
            Fields.NUMBER: "2",
            Fields.PAGES: "114--122",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b82",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b82",
            Fields.AUTHOR: "Paré, G. and Cameron, A-F and Poba-Nzaou, P.",
            Fields.TITLE: "A systematic assessment of rigor in information systems ranking-type Delphi studies",
            Fields.YEAR: "2013",
            Fields.JOURNAL: "Information & Management",
            Fields.VOLUME: "50",
            Fields.NUMBER: "5",
            Fields.PAGES: "207--217",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b83",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b83",
            Fields.AUTHOR: "Paré, G. and Tate, M. and Johnstone, D.",
            Fields.TITLE: "Contextualizing the twin concepts of systematicity and transparency in information systems literature reviews",
            Fields.YEAR: "2016",
            Fields.JOURNAL: "European Journal of Information Systems",
            Fields.VOLUME: "25",
            Fields.NUMBER: "6",
            Fields.PAGES: "493--508",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b84",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b84",
            Fields.AUTHOR: "Pinsonneault, A. and Kraemer, K.",
            Fields.TITLE: "Survey research methodology in management information systems: an assessment",
            Fields.YEAR: "1993",
            Fields.JOURNAL: "Journal of Management Information Systems",
            Fields.VOLUME: "10",
            Fields.NUMBER: "2",
            Fields.PAGES: "75--105",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b85",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b85",
            Fields.AUTHOR: "Prester, J. and Wagner, G. and Schryen, G.",
            Fields.TITLE: "Classifying the ideational impact of Information Systems review articles: a content-enriched deep learning approach",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "Decision Support Systems",
            Fields.VOLUME: "140",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b86",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b86",
            Fields.AUTHOR: "Raganato, A. and Bovi, C. D and Navigli, R.",
            Fields.TITLE: "Neural sequence learning models for word sense disambiguation",
            Fields.BOOKTITLE: "Proceedings of the conference on empirical methods in natural language processing",
            Fields.YEAR: "2017",
            Fields.PAGES: "1156--1167",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b87",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b87",
            Fields.AUTHOR: "Rai, A.",
            Fields.TITLE: "Editor's comments: synergies between big data and theory",
            Fields.YEAR: "2016",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "40",
            Fields.NUMBER: "2",
            Fields.PAGES: "iii--ix",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b88",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b88",
            Fields.AUTHOR: "Raisch, S. and Krakowski, S.",
            Fields.TITLE: "Artificial intelligence and management: the automation-augmentation paradox",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "Academy of Management Review",
            Fields.VOLUME: "46",
            Fields.NUMBER: "1",
            Fields.PAGES: "192--210",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b89",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b89",
            Fields.AUTHOR: "Rivard, S.",
            Fields.TITLE: "Editor's comments: the ions of theory construction",
            Fields.YEAR: "2014",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "38",
            Fields.NUMBER: "2",
            Fields.PAGES: "iii--xiv",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b90",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b90",
            Fields.AUTHOR: "Russell, S. J and Norvig, P.",
            Fields.TITLE: "Artificial Intelligence: A Modern Approach",
            Fields.YEAR: "2016",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b91",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b91",
            Fields.AUTHOR: "Shokraneh, Russell-Rose T And and , F.",
            Fields.TITLE: "Dsearch: Facilitating reproducible and valid searching in evidence synthesis",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "BMJ Evidence-Based Medicine",
            Fields.VOLUME: "24",
            Fields.NUMBER: "2",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b92",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b92",
            Fields.AUTHOR: "Schmiedel, T. and Müller, O. and Vom Brocke, J.",
            Fields.TITLE: "Topic modeling as a strategy of inquiry in organizational research: A tutorial with an application example on organizational culture",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "Organizational Research Methods",
            Fields.VOLUME: "22",
            Fields.NUMBER: "4",
            Fields.PAGES: "941--968",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b93",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b93",
            Fields.AUTHOR: "Schryen, G. and Wagner, G. and Benlian, A.",
            Fields.TITLE: "A knowledge development perspective on literature reviews: validation of a new typology in the IS field",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "Communications of the Association for Information Systems",
            Fields.VOLUME: "46",
            Fields.PAGES: "134--168",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b94",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b94",
            Fields.AUTHOR: "Seeber, I. and Bittner, E. and Briggs, R. O",
            Fields.TITLE: "Machines as teammates: a research agenda on AI in team collaboration",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "Information & Management",
            Fields.VOLUME: "57",
            Fields.NUMBER: "2",
            Fields.PAGES: "1--22",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b95",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b95",
            Fields.AUTHOR: "Shamliyan, T. and Kane, R. L and Dickinson, S.",
            Fields.TITLE: "A systematic review of tools used to assess the quality of observational studies that examine incidence or prevalence and risk factors for diseases",
            Fields.YEAR: "2010",
            Fields.JOURNAL: "Journal of Clinical Epidemiology",
            Fields.VOLUME: "63",
            Fields.NUMBER: "10",
            Fields.PAGES: "1061--1070",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b96",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b96",
            Fields.AUTHOR: "Sidorova, A. and Evangelopoulos, N. and Valacich, J. S",
            Fields.TITLE: "Uncovering the intellectual core of the information systems discipline",
            Fields.YEAR: "2008",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "32",
            Fields.NUMBER: "3",
            Fields.PAGES: "467--482",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b97",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b97",
            Fields.AUTHOR: "Small, H. G",
            Fields.TITLE: "Cited documents as concept symbols",
            Fields.YEAR: "1978",
            Fields.JOURNAL: "Social Studies of Science",
            Fields.VOLUME: "8",
            Fields.NUMBER: "3",
            Fields.PAGES: "327--340",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b98",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b98",
            Fields.AUTHOR: "Stanovsky, G. and Eckle-Kohler, J. and Puzikov, Y.",
            Fields.TITLE: "Integrating deep linguistic features in factuality prediction over unified datasets",
            Fields.BOOKTITLE: "Proceedings of the Annual Meeting of the Association for Computational Linguistics",
            Fields.YEAR: "2017",
            Fields.PAGES: "352--357",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b99",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b99",
            Fields.AUTHOR: "Sturm, B. and Sunyaev, A.",
            Fields.TITLE: "Design principles for systematic search systems: a holistic synthesis of a rigorous multi-cycle design science research journey",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Business & Information Systems Engineering",
            Fields.VOLUME: "61",
            Fields.NUMBER: "1",
            Fields.PAGES: "91--111",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b100",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b100",
            Fields.AUTHOR: "Swanson, D. R and Smalheiser, N. R",
            Fields.TITLE: "An interactive system for finding complementary literatures: a stimulus to scientific discovery",
            Fields.YEAR: "1997",
            Fields.JOURNAL: "Artificial Intelligence",
            Fields.VOLUME: "91",
            Fields.NUMBER: "2",
            Fields.PAGES: "183--203",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b101",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b101",
            Fields.AUTHOR: "Tate, W. L and Ellram, L. M and Kirchoff, J. F",
            Fields.TITLE: "Corporate social responsibility reports: a thematic analysis related to supply chain management",
            Fields.YEAR: "2010",
            Fields.JOURNAL: "Journal of Supply Chain Management",
            Fields.VOLUME: "46",
            Fields.NUMBER: "1",
            Fields.PAGES: "19--44",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b102",
            Fields.ENTRYTYPE: ENTRYTYPES.BOOK,
            Fields.TEI_ID: "b102",
            Fields.AUTHOR: "Taulli, T. and Oni, M.",
            Fields.TITLE: "Artificial Intelligence Basics. 1st edition",
            Fields.YEAR: "2019",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b103",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b103",
            Fields.AUTHOR: "Templier, M. and Paré, G.",
            Fields.TITLE: "Transparency in literature reviews: an assessment of reporting practices across review types and genres in top IS journals",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "European Journal of Information Systems",
            Fields.VOLUME: "27",
            Fields.NUMBER: "5",
            Fields.PAGES: "503--550",
            Fields.NR_INTEXT_CITATIONS: 15,
        },
        {
            Fields.ID: "b104",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b104",
            Fields.AUTHOR: "Thilakaratne, M. and Falkner, K. and Atapattu, T.",
            Fields.TITLE: "A systematic review on literature-based discovery: general overview, methodology, & statistical analysis",
            Fields.YEAR: "2019",
            Fields.JOURNAL: "ACM Computing Surveys",
            Fields.VOLUME: "52",
            Fields.NUMBER: "6",
            Fields.PAGES: "1--34",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b105",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b105",
            Fields.AUTHOR: "Thomas, J. and Noel-Storr, A. and Marshall, I.",
            Fields.TITLE: "Living systematic reviews: 2. Combining human and machine effort",
            Fields.YEAR: "2017",
            Fields.JOURNAL: "Journal of Clinical Epidemiology",
            Fields.VOLUME: "91",
            Fields.PAGES: "31--37",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b106",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b106",
            Fields.AUTHOR: "Tkaczyk, D. and Collins, A. and Sheridan, P.",
            Fields.TITLE: "Machine learning vs. Rules and out-of-the-box vs. retrained",
            Fields.BOOKTITLE: "Proceedings of the ACM/IEEE on Joint Conference on Digital Libraries",
            Fields.YEAR: "2018",
            Fields.NR_INTEXT_CITATIONS: 0,
        },
        {
            Fields.ID: "b107",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b107",
            Fields.AUTHOR: "Tremblay, M. C and Der Meer, Van and Beck, D. and , R.",
            Fields.TITLE: "The effects of the quantification of faculty productivity: perspectives from the design science research community",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Communications of the Association for Information Systems",
            Fields.VOLUME: "43",
            Fields.PAGES: "625--661",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b108",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b108",
            Fields.AUTHOR: "Tsafnat, G. and Glasziou, P. and Choong, M. K",
            Fields.TITLE: "Systematic review automation technologies",
            Fields.YEAR: "2014",
            Fields.JOURNAL: "Systematic Reviews",
            Fields.VOLUME: "3",
            Fields.PAGES: "1--15",
            Fields.NR_INTEXT_CITATIONS: 5,
        },
        {
            Fields.ID: "b109",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b109",
            Fields.AUTHOR: "Van De Schoot, R. and De Bruin, J. and Schram, R.",
            Fields.TITLE: "An open source machine learning framework for efficient and transparent systematic reviews",
            Fields.YEAR: "2021",
            Fields.JOURNAL: "Nature Machine Intelligence",
            Fields.VOLUME: "3",
            Fields.PAGES: "125--133",
            Fields.NR_INTEXT_CITATIONS: 3,
        },
        {
            Fields.ID: "b110",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b110",
            Fields.AUTHOR: "Van Dinter, R. and Tekinerdogan, B. and Catal, C.",
            Fields.TITLE: "Automation of systematic literature reviews: a systematic literature review",
            Fields.YEAR: "2021",
            Fields.JOURNAL: "Information and Software Technology",
            Fields.VOLUME: "136",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b111",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b111",
            Fields.AUTHOR: "Vom Brocke, J. and Simons, A. and Riemer, K.",
            Fields.TITLE: "Standing on the shoulders of giants: challenges and recommendations of literature search in information systems research. Communications of the Association for",
            Fields.YEAR: "2015",
            Fields.JOURNAL: "Information Systems",
            Fields.VOLUME: "37",
            Fields.NUMBER: "9",
            Fields.PAGES: "205--224",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b112",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b112",
            Fields.AUTHOR: "Wahyudi, A. and Kuk, G. and Janssen, M.",
            Fields.TITLE: "A process pattern model for tackling and improving big data quality",
            Fields.YEAR: "2018",
            Fields.JOURNAL: "Information Systems Frontiers",
            Fields.VOLUME: "20",
            Fields.NUMBER: "3",
            Fields.PAGES: "457--469",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b113",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b113",
            Fields.AUTHOR: "Wallace, B. C and Small, K. and Brodley, C. E",
            Fields.TITLE: "Deploying an interactive machine learning system in an evidence-based practice center: Abstrackr",
            Fields.BOOKTITLE: "Proceedings of the ACM SIGHIT International Health Informatics Symposium",
            Fields.YEAR: "2012",
            Fields.PAGES: "819--824",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b114",
            Fields.ENTRYTYPE: ENTRYTYPES.INPROCEEDINGS,
            Fields.TEI_ID: "b114",
            Fields.AUTHOR: "Wand, Y. and Weber, R.",
            Fields.TITLE: "An ontological analysis of some fundamental information systems concepts",
            Fields.BOOKTITLE: "Proceedings of the International Conference on Information Systems",
            Fields.YEAR: "1988",
            Fields.PAGES: "213--226",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b115",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b115",
            Fields.AUTHOR: "Wang, Z. and Nayfeh, T. and Tetzlaff, J.",
            Fields.TITLE: "Error rates of human reviewers during abstract screening in systematic reviews",
            Fields.YEAR: "2020",
            Fields.JOURNAL: "PLoS One",
            Fields.VOLUME: "15",
            Fields.NUMBER: "1",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b116",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b116",
            Fields.AUTHOR: "Van Zoonen, W. and Der Meer, Van and , Tgla",
            Fields.TITLE: "Social media research: the application of supervised machine learning in organizational communication research",
            Fields.YEAR: "2016",
            Fields.JOURNAL: "Computers in Human Behavior",
            Fields.VOLUME: "63",
            Fields.PAGES: "132--141",
            Fields.NR_INTEXT_CITATIONS: 0,
        },
        {
            Fields.ID: "b117",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b117",
            Fields.AUTHOR: "Webster, J. and Watson, R. T",
            Fields.TITLE: "Analyzing the past to prepare for the future: writing a literature review",
            Fields.YEAR: "2002",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "26",
            Fields.NUMBER: "2",
            Fields.PAGES: "xiii--xxiii",
            Fields.NR_INTEXT_CITATIONS: 2,
        },
        {
            Fields.ID: "b118",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b118",
            Fields.AUTHOR: "Whetten, D. A",
            Fields.TITLE: "What constitutes a theoretical contribution?",
            Fields.YEAR: "1989",
            Fields.JOURNAL: "Academy of Management Review",
            Fields.VOLUME: "14",
            Fields.NUMBER: "4",
            Fields.PAGES: "490--495",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
        {
            Fields.ID: "b119",
            Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
            Fields.TEI_ID: "b119",
            Fields.AUTHOR: "Xu, J. and Benbasat, I. and Benbasat, I.",
            Fields.TITLE: "Integrating service quality with system and information quality: an empirical test in the e-service context",
            Fields.YEAR: "2013",
            Fields.JOURNAL: "MIS Quarterly",
            Fields.VOLUME: "37",
            Fields.NUMBER: "3",
            Fields.PAGES: "777--794",
            Fields.NR_INTEXT_CITATIONS: 1,
        },
    ] == tei_doc.get_references(add_intext_citation_count=True)


def test_tei_citations_per_section(tei_doc, tmp_path) -> None:  # type: ignore
    """Test the tei citations per section method."""

    assert {
        "introduction": [
            "b0",
            "b108",
            "b50",
            "b45",
            "b107",
            "b69",
            "b45",
            "b59",
            "b93",
            "b59",
            "b94",
            "b90",
            "b102",
            "b32",
            "b74",
            "b70",
            "b96",
            "b75",
            "b59",
        ],
        "artificial intelligence-based support for the literature review process": [
            "b108",
            "b103",
            "b3",
            "b35",
            "b47",
            "b55",
            "b72",
            "b108",
            "b110",
            "b73",
            "b103",
            "b76",
            "b89",
            "b50",
            "b104",
            "b63",
            "b2",
            "b29",
            "b100",
            "b5",
            "b103",
            "b20",
            "b83",
            "b103",
            "b21",
            "b63",
            "b14",
            "b99",
            "b16",
            "b45",
            "b59",
            "b103",
            "b103",
            "b35",
            "b109",
            "b59",
            "b59",
            "b63",
            "b16",
            "b115",
            "b38",
            "b51",
            "b103",
            "b103",
            "b36",
            "b11",
            "b73",
            "b103",
            "b47",
            "b103",
            "b93",
            "b54",
            "b8",
            "b77",
            "b104",
            "b61",
            "b101",
            "b13",
            "b64",
            "b78",
            "b11",
            "b57",
            "b75",
            "b92",
            "b6",
            "b13",
            "b45",
            "b118",
        ],
        "level i: supporting infrastructure": [
            "b59",
            "b23",
            "b6",
            "b10",
            "b49",
            "b68",
            "b112",
            "b91",
            "b27",
            "b1",
            "b66",
            "b63",
            "b111",
            "b71",
            "b114",
            "b111",
            "b21",
            "b81",
            "b103",
            "b99",
            "b22",
            "b31",
            "b56",
        ],
        "level ii: methods and tools": [
            "b87",
            "b30",
            "b48",
            "b68",
            "b76",
            "b108",
            "b44",
            "b81",
            "b103",
            "b91",
            "b59",
            "b80",
            "b109",
            "b113",
            "b105",
            "b108",
            "b103",
            "b103",
            "b38",
            "b41",
            "b73",
            "b95",
            "b84",
            "b25",
            "b82",
            "b47",
            "b7",
            "b52",
            "b59",
            "b85",
            "b63",
            "b37",
            "b97",
            "b24",
            "b86",
            "b98",
            "b115",
            "b119",
            "b58",
            "b59",
            "b85",
            "b34",
            "b63",
            "b62",
            "b89",
            "b117",
            "b13",
            "b19",
            "b78",
            "b40",
            "b63",
            "b65",
            "b83",
            "b80",
            "b58",
            "b85",
            "b60",
            "b60",
            "b15",
            "b83",
            "b103",
            "b21",
            "b18",
            "b39",
            "b42",
            "b4",
            "b26",
            "b17",
            "b43",
            "b46",
            "b21",
            "b18",
            "b33",
            "b53",
            "b12",
            "b3",
            "b30",
            "b79",
            "b72",
            "b109",
            "b88",
            "b94",
        ],
        "level iii: research practice": [
            "b96",
            "b28",
            "b58",
            "b7",
            "b52",
            "b67",
            "b69",
            "b15",
            "b69",
            "b15",
        ],
        "concluding remarks": ["b62", "b89", "b117"],
    } == tei_doc.get_citations_per_section()


def test_tei_mark_references(tei_doc, tmp_path) -> None:  # type: ignore
    """Test the tei extraction of references"""

    # change tei_path to prevent changes to the original tei file
    tei_doc.tei_path = tmp_path / Path("test.tei.xml")
    tei_doc.mark_references(
        records={
            "TEST_ID": {
                Fields.ID: "TEST_ID",
                Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
                Fields.STATUS: RecordState.rev_included,
                Fields.TITLE: "What constitutes a theoretical contribution?",
                Fields.JOURNAL: "Academy of Management Review",
                Fields.AUTHOR: "Whetten, D. A",
                Fields.YEAR: "1989",
                Fields.VOLUME: "14",
                Fields.NUMBER: "4",
            },
            "NO_TITLE": {
                Fields.ID: "NO_TITLE",
                Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
                Fields.STATUS: RecordState.rev_included,
                Fields.JOURNAL: "Academy of Management Review",
                Fields.AUTHOR: "Whetten, D. A",
                Fields.YEAR: "1989",
                Fields.VOLUME: "14",
                Fields.NUMBER: "4",
            },
            "NOT_INCLUDED": {
                Fields.ID: "NOT_INCLUDED",
                Fields.ENTRYTYPE: ENTRYTYPES.ARTICLE,
                Fields.STATUS: RecordState.rev_prescreen_excluded,
                Fields.JOURNAL: "Academy of Management Review",
                Fields.AUTHOR: "Whetten, D. A",
                Fields.YEAR: "1989",
                Fields.VOLUME: "14",
                Fields.NUMBER: "4",
            },
        }
    )
    actual = tei_doc.get_tei_str()
    expected = '<biblStruct xml:id="b118" ID="TEST_ID">'
    assert expected in actual

    expected = 'theoretical rationale <ref type="bibr" target="#b118" ID="TEST_ID">(Whetten, 1989)</ref>, which is critical '
    assert expected in actual

    assert "NO_TITLE" not in actual

    assert "NOT_INCLUDED" not in actual


def test_tei_exception(tmp_path) -> None:  # type: ignore
    tei_path = tmp_path / Path("erroneous_tei.tei.xml")

    with open(tei_path, "wb") as f:
        f.write(b"[BAD_INPUT_DATA]")

    environment_manager = colrev.env.environment_manager.EnvironmentManager()

    with pytest.raises(colrev_exceptions.TEIException):
        colrev.env.tei_parser.TEIParser(
            environment_manager=environment_manager, tei_path=tei_path
        )


def test_tei_pdf_not_exists() -> None:
    pdf_path = Path("data/non_existent.pdf")

    environment_manager = colrev.env.environment_manager.EnvironmentManager()

    with pytest.raises(FileNotFoundError):
        colrev.env.tei_parser.TEIParser(
            environment_manager=environment_manager, pdf_path=pdf_path
        )
